

/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */



#include "gtkhelpers.h"
#include <stdlib.h>


int menuopt0=0;
int menuopt1=1;
int menuopt2=2;
int menuopt3=3;
int menuopt4=4;
int menuopt5=5;
int menuopt6=6;
int menuopt7=7;
int menuopt8=8;
int menuopt9=9;
int menuopt10=10;
int menuopt11=11;
int menuopt12=12;
int menuopt13=13;
int menuopt14=14;
int menuopt15=15;
int menuopt16=16;
int menuopt17=17;
int menuopt18=18;
int menuopt19=19;
int menuopt20=20;



/* This function creates a random string. Gtk uses an internal
   reference string for dereferencing a widget, and with many widgets
   the creation of unique internal references may becoms quite annoying.
   So if a NULL pointer is passed as intref, a random string will be generated.
*/

/* TODO: We need to check whether Gtk uses the pointer or strcpy's
   the string. In the latter case, we need to free the pointer or
   we have a small but ugly memory leak per widget */

gchar * rnd_intref ()
{
gchar *rndstr;
int i;

	rndstr = calloc (9, sizeof(gchar));		/* 8-length plus terminating zero */
	for (i=0; i<8; i++) rndstr[i]=0x20 + (rand() & 0x5f);
	return rndstr;
}


/****************************************************************************/


GtkWidget* Menuitem (GtkWidget* toplevel, GtkWidget* target, gpointer signal, const gchar* label, const gchar* intref)
{
GtkWidget* item;

	if (intref==NULL) intref = rnd_intref();
	item = gtk_menu_item_new_with_label (label);
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (target), item);
	if (signal) gtk_signal_connect (GTK_OBJECT (item), "activate",
                      GTK_SIGNAL_FUNC (signal), NULL);

	return item;
}


GtkWidget* Makesubmenu (GtkWidget* toplevel, GtkWidget* target, const gchar* intref)
{
GtkWidget* item;

	if (intref==NULL) intref = rnd_intref();
  	item = gtk_menu_new ();
  	gtk_menu_item_set_submenu (GTK_MENU_ITEM (target), item);
  	return item;
}


/* Make_Optionmenu creates an option menu and attaches it to a table */

GtkWidget* Make_Optionmenu (GtkWidget* toplevel, GtkWidget* target, const gchar* intref, int x, int y)
{
GtkWidget* myMenu;

	if (intref==NULL) intref = rnd_intref();
	myMenu = gtk_option_menu_new ();
	gtk_widget_show (myMenu);
  	if ((x>=0) && (y>=0))	/* Assume we want to attch to a table */
  	{
		gtk_table_attach (GTK_TABLE (target), myMenu, x,x+1,y,y+1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);
	}
	else		/* Assume we want to attach to a vbox */
	{
		gtk_box_pack_start (GTK_BOX (target), myMenu, FALSE, FALSE, 3);
	}
	return myMenu;
}
 
/* Option_Menuitem creates one option in an optionmenu */

GtkWidget* Option_menuitem (GtkWidget* target, 
						gpointer signal, const gchar* label, gpointer optionvalue)
{
GtkWidget* item;

	item = gtk_menu_item_new_with_label (label);
	gtk_widget_show (item);
	gtk_menu_append (GTK_MENU (target), item);
	if (signal) gtk_signal_connect (GTK_OBJECT(item),"activate",GTK_SIGNAL_FUNC(signal), optionvalue);
	return item;

}


/* MakeLabel creates a label widget and attaches it to a 
	table at (x,y) in window toplevel (if x>=0 and y>=0), or
	to a vbox/hbox if x<0 or y<0 */

GtkWidget* MakeLabel (GtkWidget* toplevel, GtkWidget* target, const gchar* lbltxt, 
					  const gchar* intref, int tblx, int tbly)
{
GtkWidget* thislabel;

	if (intref==NULL) intref = rnd_intref();
	thislabel = gtk_label_new (lbltxt);
	gtk_widget_show (thislabel);
	gtk_label_set_justify (GTK_LABEL (thislabel), GTK_JUSTIFY_LEFT);

	if ((tblx>=0) && (tbly>=0))	/* Assume we want to attch to a tbale */
	{
  	  gtk_table_attach (GTK_TABLE (target), thislabel, tblx, tblx+1, tbly, tbly+1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);
  	  gtk_misc_set_padding (GTK_MISC (thislabel), 8, 5);
	}
	else		/* Assume we want to attach to a vbox */
	{
  	  gtk_box_pack_start (GTK_BOX (target), thislabel, FALSE, FALSE, 3);
	}
	return thislabel;
}


/* MakeHsep creates a H-separator widget and attaches it to a 
	table at (x,y) in window toplevel (if x>=0 and y>=0), or
	to a vbox/hbox if x<0 or y<0 */

GtkWidget* MakeHsep (GtkWidget* toplevel, GtkWidget* target, 
					  const gchar* intref, int tblx, int tbly)
{
GtkWidget* this_hseparator;

	if (intref==NULL) intref = rnd_intref();
	this_hseparator = gtk_hseparator_new ();
	gtk_widget_show (this_hseparator);
	
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), this_hseparator, tblx,tblx+1,tbly,tbly+1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	}
	else
	{
		gtk_box_pack_start (GTK_BOX (target), this_hseparator, FALSE, FALSE, 2);
	}
	return this_hseparator;
}


GtkWidget* MakeVsep (GtkWidget* toplevel, GtkWidget* target, 
					  const gchar* intref, int tblx, int tbly)
{
GtkWidget* this_vseparator;

	if (intref==NULL) intref = rnd_intref();
	this_vseparator = gtk_vseparator_new ();
	gtk_widget_show (this_vseparator);
	
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), this_vseparator, tblx,tblx+1,tbly,tbly+1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	}
	else
	{
		gtk_box_pack_start (GTK_BOX (target), this_vseparator, FALSE, FALSE, 2);
	}
	return this_vseparator;
}



/* MakeHscale creates a h-scale from an adjustment and attaches it to a 
	table at (x,y) in window toplevel (if x>=0 and y>=0), or
	to a vbox/hbox if x<0 or y<0 */

GtkWidget* MakeHScale (GtkAdjustment* adjust, GtkWidget* toplevel, GtkWidget* target,
					  const gchar* intref, int tblx, int tbly)

{
GtkWidget* hscale;

	if (intref==NULL) intref = rnd_intref();
	hscale = gtk_hscale_new (GTK_ADJUSTMENT (adjust));
	gtk_widget_show (hscale);
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), hscale, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 10, 4);
	}
	else
	{
		gtk_box_pack_start (GTK_BOX (target), hscale, TRUE, TRUE, 2);
	}
	
	return hscale;
}


/* MakeSpinbutton as above, but creates a spin button and attaches it to a 
	table at (x,y) in window toplevel (if x>=0 and y>=0), or
	to a vbox/hbox if x<0 or y<0 */

GtkWidget* MakeSpinbutton (GtkAdjustment* adjust, GtkWidget* toplevel, GtkWidget* target,
					  const gchar* intref, int tblx, int tbly)

{
GtkWidget* spinner;

	if (intref==NULL) intref = rnd_intref();
	spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjust),1,0);
	gtk_widget_show (spinner);
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), spinner, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 10, 4);
	}
	else
	{
		gtk_box_pack_start (GTK_BOX (target), spinner, TRUE, TRUE, 2);
	}

  return spinner;
}


/* Button_in_table is just that - it attaches a button to a table.
	Not quite accurate - if x<0 or y<0 it's a vbox/hbox.
   If a signal function pointer is provided, it will be attached
   with the "clicked" signal, passing toplevel as user_data argument */

GtkWidget* Button_in_table (GtkWidget* toplevel, GtkWidget* target, const gchar* caption, 
					  const gchar* intref, int tblx, int tbly, gpointer signal)
{
GtkWidget* thisbutton;

	if (intref==NULL) intref = rnd_intref();
	thisbutton = gtk_button_new_with_label (caption);
	gtk_widget_show (thisbutton);
	
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), thisbutton, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	}	
	else
	{
		gtk_box_pack_start (GTK_BOX (target), thisbutton, TRUE, TRUE, 2);
	}
	
	if (signal) gtk_signal_connect (GTK_OBJECT(thisbutton),"clicked",GTK_SIGNAL_FUNC(signal), toplevel);
 
	return thisbutton;
}


/* In a similar way we create toggle buttons */

GtkWidget* Togglebutton_in_table (GtkWidget* toplevel, GtkWidget* target, const gchar* caption, 
					  const gchar* intref, int tblx, int tbly, gpointer signal)
{
GtkWidget* thisbutton;

	if (intref==NULL) intref = rnd_intref();
	thisbutton = gtk_toggle_button_new_with_label (caption);
	gtk_widget_show (thisbutton);
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), thisbutton, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	}	
	else
	{
		gtk_box_pack_start (GTK_BOX (target), thisbutton, TRUE, TRUE, 2);
	}

	if (signal) gtk_signal_connect (GTK_OBJECT(thisbutton),"toggled",GTK_SIGNAL_FUNC(signal), toplevel);
 
  
	return thisbutton;
}


/* In a similar way we create check buttons. Getting boring at this point? */

GtkWidget* Checkbutton_in_table (GtkWidget* toplevel, GtkWidget* target, const gchar* caption,
					  const gchar* intref, int tblx, int tbly, gpointer signal)
{
GtkWidget* thisbutton;

	if (intref==NULL) intref = rnd_intref();
	thisbutton = gtk_check_button_new_with_label (caption);
	gtk_widget_show (thisbutton);
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), thisbutton, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	}	
	else
	{
		gtk_box_pack_start (GTK_BOX (target), thisbutton, TRUE, TRUE, 2);
	}

	if (signal) gtk_signal_connect (GTK_OBJECT(thisbutton),"toggled",GTK_SIGNAL_FUNC(signal), toplevel);
 
	return thisbutton;
}


/* Another guy of the same kind. Here, it is an entry box. No signal, though */

GtkWidget* Entry_in_table (GtkWidget* toplevel, GtkWidget* target, const gchar* caption, 
					  const gchar* intref, int tblx, int tbly)
{
GtkWidget* thisentry;

	if (intref==NULL) intref = rnd_intref();
	thisentry = gtk_entry_new ();
	gtk_widget_show (thisentry);
	if ((tblx>=0) && (tbly>=0))
	{
		gtk_table_attach (GTK_TABLE (target), thisentry, tblx, tblx+1, tbly, tbly+1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 4, 4);
	}	
	else
	{
		gtk_box_pack_start (GTK_BOX (target), thisentry, TRUE, TRUE, 2);
	}



	if (caption) gtk_entry_set_text (GTK_ENTRY(thisentry), caption);
 
	return thisentry;
}




/* Often, we define a vbox as the first container in a window.
   This macro performs just that... */
   
   
GtkWidget* vbox_in_window (GtkWidget* toplevel, gchar* intref)
{
GtkWidget* vbox;
	 
	if (intref==NULL) intref = rnd_intref();
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (toplevel), vbox);
	return vbox;
}


/* ... as well as a hbox inside that vbox... */


GtkWidget* hbox_in_window (GtkWidget* toplevel, GtkWidget* target, gchar* intref)
{
GtkWidget* hbox;
	 
	if (intref==NULL) intref = rnd_intref();
	hbox = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (target), hbox, TRUE, TRUE, 0);
	return hbox;
}


	

GtkWidget* MakeTable (GtkWidget* toplevel, GtkWidget* target, gchar* intref, int sizex, int sizey)
{
GtkWidget* MyTable;

	if (intref==NULL) intref = rnd_intref();
	MyTable = gtk_table_new (sizey, sizex, FALSE);  /* sic! rows x columns */
	gtk_widget_show (MyTable);
	gtk_box_pack_start (GTK_BOX (target), MyTable, FALSE, FALSE, 0);
	return MyTable;
}


/* Add a tooltip to a widget. This function creates the tooltip widget and returns a pointer to it */

GtkTooltips* MakeTooltip (GtkWidget *target_widget, gchar *tooltip_text)
{
GtkTooltips *tt;

	tt = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tt, target_widget, tooltip_text, NULL);
	return tt;
}








