

/*
 *	VIZ: Visualization functions.
 */


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "application.h"
#include "backgnd.h"
#include "callbacks.h"
#include "defs.h"
#include "dlgbox.h"
#include "filter.h"
#include "findclust.h"
#include "functions.h"
#include "graphdsp.h"
#include "gtkhelpers.h"
#include "imgbuf.h"
#include "interface.h"
#include "morphop.h"
#include "roi.h"
#include "secondwnd.h"
#include "statop.h"
#include "special1.h"
#include "tiffstuff.h"
#include "viz.h"




/**********************************************************

	Visualization tools

	This module is a collection of visualization tools,
	beginning from simple brightness/contrast manipulation
	to surface rendering. Soecifically, it provides

	1) Brightness - contrast - gamma manipulation
	2) Predefined ("canned" color maps
	3) Freehand color maps
	4) 3D surface plots



/**********************************************************/




/* bcg is a quick brightness-contrast-gamma mapping function.
	Input range is 0 <= f <= 1; output range is the same (preserve sign, symmetric)
	Uses viz_bright, viz_contrast, viz_gamma
*/


float bcg (float f)
{
int isign; float v;

// 	dp (2, "Converting %f",f);
	isign = (f<0.0? -1: 1);
	v = fabs (f);

	if (viz_gamma!=1.0) v = pow (v, viz_gamma);		/* Gamma is first. Raise value to the power of viz_gamma */
	v = 0.5 + viz_contrast*(v+viz_bright-0.5);		/* Contrast + Brightness in one step */
	if (v<0.0) v=0.0; else if (v>1.0) v=1.0;
// 	dp (2, " to %f\n",isign*v);

	return isign*v;
}





/*          ###     ###    #        ###    ####    #   #     #     ####     ###           */
/*         #   #   #   #   #       #   #   #   #   ## ##    # #    #   #   #   #          */
/*         #       #   #   #       #   #   #   #   # # #   #   #   #   #   #              */
/*         #       #   #   #       #   #   ####    # # #   #   #   ####     ###           */
/*         #       #   #   #       #   #   # #     #   #   #####   #           #          */
/*         #   #   #   #   #       #   #   #  #    #   #   #   #   #       #   #          */
/*          ###     ###    #####    ###    #   #   #   #   #   #   #        ###           */


/* 	The colormap functions are pixel-by-pixel translation functions. They receive two
	parameters: a luminance value between 0 and 1, and a zero flag which is set when
	the unscaled image value is zero (for the k colormaps).
	They return a rgbtriplet for storage in the imlib buffer */


/* A pointer to the currently active colormap function */

rgbtriplet (*cmap_function)(float l, unsigned char iszero) = NULL;



void reset_colormap ()			/* Mark color maps as unused */
{
	cmap_function = NULL;
	active_cmap=0;
	cmap_use_contours=0;
	cmap_use_logviz = 0;
}



void init_cmap_lut ()		/* Reset LUT to default */
{
int i;

	for (i=0; i<256; i++)
	{
		redlut[i]=i; 
		greenlut[i]=i; 
		bluelut[i]=i; 
	}
}



/* The default colormap. out = in. */

rgbtriplet cmap_grayscale (float l, unsigned char iszero)
{
rgbtriplet rgb;

	rgb.red = (int)(l*255.0);
	rgb.green=rgb.blue=rgb.red;
	return (rgb);

}



/* A very simple LUT to show the image's negative */

rgbtriplet cmap_negative (float l, unsigned char iszero)
{
rgbtriplet rgb;

	rgb.red = (int)((1.0-l)*255.0);
	rgb.green=rgb.blue=rgb.red;
	return (rgb);

}


/* A simple general-purpose LUT where the sum of R+B+G is always 1.
	If (iszero) is used, a zero-valued pixel returns black */

rgbtriplet cmap_rgb_simple (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = IROUND(255.0*l);
	if (l<=0.5)
	{
		rgb.blue=255-2*i;
		rgb.green = 2*i;
		rgb.red=0;
	}
	else
	{
		rgb.blue=0;
		rgb.green = 255-2*(i-128);
		rgb.red= 1 + 2*(i-128);
	}

	return (rgb);

}


rgbtriplet cmap_rgbk_simple (float l, unsigned char iszero)
{

	if (iszero) return (rgbtriplet){0,0,0};
	return cmap_rgb_simple (l,1);
}

/* The famous "rainbow" function, covers the color gamut from blue to red
	with full color saturation */

rgbtriplet cmap_rainbow (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = (int)(255.0*l);
	if (i<64)
	{
		rgb.blue=255;
		rgb.green = 4*i;
		rgb.red=0;
	}
	else if (i<128)
	{
		rgb.blue=511-4*i;
		rgb.green = 255;
		rgb.red= 0;
	}
	else if (i<192)
	{
		rgb.blue=0;
		rgb.green = 255;
		rgb.red= 4*i-512;
	}
	else
	{
		rgb.blue=0;
		rgb.green = 1023-4*i;
		rgb.red= 255;
	}

	return (rgb);

}


rgbtriplet cmap_rainbow_k (float l, unsigned char iszero)
{

	if (iszero) return (rgbtriplet){0,0,0};
	return cmap_rainbow (l,1);
}


/* A great general-purpose LUT "lifted" from NIH image,
   it shows dark shades in hues of black and blue, then
   progresses over magenta, red, orange, yellow
   to white. */

rgbtriplet cmap_fire (float l, unsigned char iszero)
{
int i; rgbtriplet rgb;

	i = (int)(255.0*l);
	rgb = (rgbtriplet){0,0,0};
		
	if (i<64) rgb.blue=4*i;
	else if (i<128) rgb.blue=511-4*i;
	else if (i>212) rgb.blue=6*i-1275;
	
	if ((i>=16) && (i<=143)) rgb.red=2*i-32;
	else if (i>143) rgb.red=255;
	
	if ((i>100) && (i<=227)) rgb.green=2*i-200;
	else if (i>227) rgb.green=255;

	return (rgb);

}



/* A color map for terrain-like colors - "lifted" from terraform.
   This color map creates a banded structure with relatively steep gradients.
	It is a real LUT in the sense that we need to round the value l
	and use a lookup table */

void cmap_terra_prep ()
{
int i;	
rgbtriplet terracolors[256] = {
									{	0	,	0	,	0	},					{	4	,	4	,	18	},
									{	8	,	8	,	37	},					{	12	,	12	,	56	},
									{	16	,	16	,	74	},					{	20	,	20	,	93	},
									{	25	,	25	,	112	},					{	25	,	25	,	112	},
									{	25	,	25	,	112	},					{	25	,	25	,	112	},
									{	25	,	25	,	112	},					{	25	,	25	,	112	},
									{	25	,	25	,	112	},					{	25	,	25	,	112	},
									{	25	,	25	,	112	},					{	25	,	25	,	112	},
									{	25	,	25	,	112	},					{	20	,	20	,	127	},
									{	16	,	16	,	143	},					{	12	,	12	,	158	},
									{	8	,	8	,	174	},					{	4	,	4	,	189	},
									{	0	,	0	,	205	},					{	0	,	0	,	205	},
									{	0	,	0	,	205	},					{	0	,	0	,	205	},
									{	0	,	0	,	205	},					{	0	,	0	,	205	},
									{	0	,	0	,	205	},					{	0	,	0	,	205	},
									{	0	,	0	,	205	},					{	0	,	0	,	205	},
									{	0	,	0	,	205	},					{	0	,	0	,	213	},
									{	0	,	0	,	221	},					{	0	,	0	,	230	},
									{	0	,	0	,	238	},					{	0	,	0	,	246	},
									{	0	,	0	,	255	},					{	0	,	0	,	255	},
									{	0	,	0	,	255	},					{	0	,	0	,	255	},
									{	0	,	0	,	255	},					{	0	,	0	,	255	},
									{	0	,	0	,	255	},					{	0	,	0	,	255	},
									{	0	,	0	,	255	},					{	0	,	0	,	255	},
									{	0	,	0	,	255	},					{	5	,	24	,	255	},
									{	10	,	48	,	255	},					{	15	,	72	,	255	},
									{	20	,	96	,	255	},					{	25	,	120	,	255	},
									{	30	,	144	,	255	},					{	30	,	144	,	255	},
									{	30	,	144	,	255	},					{	30	,	144	,	255	},
									{	30	,	144	,	255	},					{	30	,	144	,	255	},
									{	30	,	144	,	255	},					{	30	,	144	,	255	},
									{	30	,	144	,	255	},					{	30	,	144	,	255	},
									{	30	,	144	,	255	},					{	25	,	159	,	212	},
									{	20	,	175	,	170	},					{	15	,	191	,	127	},
									{	10	,	206	,	85	},					{	5	,	222	,	42	},
									{	0	,	238	,	0	},					{	0	,	238	,	0	},
									{	0	,	238	,	0	},					{	0	,	238	,	0	},
									{	0	,	238	,	0	},					{	0	,	238	,	0	},
									{	0	,	238	,	0	},					{	0	,	238	,	0	},
									{	0	,	238	,	0	},					{	0	,	238	,	0	},
									{	0	,	238	,	0	},					{	0	,	232	,	0	},
									{	0	,	227	,	0	},					{	0	,	221	,	0	},
									{	0	,	216	,	0	},					{	0	,	210	,	0	},
									{	0	,	205	,	0	},					{	0	,	205	,	0	},
									{	0	,	205	,	0	},					{	0	,	205	,	0	},
									{	0	,	205	,	0	},					{	0	,	205	,	0	},
									{	0	,	205	,	0	},					{	0	,	205	,	0	},
									{	0	,	205	,	0	},					{	0	,	205	,	0	},
									{	0	,	205	,	0	},					{	0	,	213	,	0	},
									{	0	,	221	,	0	},					{	0	,	230	,	0	},
									{	0	,	238	,	0	},					{	0	,	246	,	0	},
									{	0	,	255	,	0	},					{	0	,	255	,	0	},
									{	0	,	255	,	0	},					{	0	,	255	,	0	},
									{	0	,	255	,	0	},					{	0	,	255	,	0	},
									{	0	,	255	,	0	},					{	0	,	255	,	0	},
									{	0	,	255	,	0	},					{	0	,	255	,	0	},
									{	0	,	255	,	0	},					{	8	,	243	,	2	},
									{	16	,	231	,	5	},					{	24	,	219	,	8	},
									{	32	,	207	,	10	},					{	40	,	195	,	13	},
									{	48	,	184	,	16	},					{	48	,	184	,	16	},
									{	48	,	184	,	16	},					{	48	,	184	,	16	},
									{	48	,	184	,	16	},					{	48	,	184	,	16	},
									{	48	,	184	,	16	},					{	48	,	184	,	16	},
									{	48	,	184	,	16	},					{	48	,	184	,	16	},
									{	48	,	184	,	16	},					{	58	,	176	,	23	},
									{	68	,	169	,	31	},					{	79	,	161	,	38	},
									{	89	,	154	,	46	},					{	99	,	146	,	53	},
									{	110	,	139	,	61	},					{	110	,	139	,	61	},
									{	110	,	139	,	61	},					{	110	,	139	,	61	},
									{	110	,	139	,	61	},					{	110	,	139	,	61	},
									{	110	,	139	,	61	},					{	110	,	139	,	61	},
									{	110	,	139	,	61	},					{	110	,	139	,	61	},
									{	110	,	139	,	61	},					{	131	,	152	,	72	},
									{	152	,	166	,	84	},					{	174	,	179	,	95	},
									{	195	,	193	,	107	},					{	216	,	206	,	118	},
									{	238	,	220	,	130	},					{	238	,	220	,	130	},
									{	238	,	220	,	130	},					{	238	,	220	,	130	},
									{	238	,	220	,	130	},					{	238	,	220	,	130	},
									{	238	,	220	,	130	},					{	238	,	220	,	130	},
									{	238	,	220	,	130	},					{	238	,	220	,	130	},
									{	238	,	220	,	130	},					{	232	,	215	,	127	},
									{	227	,	210	,	124	},					{	221	,	205	,	121	},
									{	216	,	200	,	118	},					{	210	,	195	,	115	},
									{	205	,	190	,	112	},					{	205	,	190	,	112	},
									{	205	,	190	,	112	},					{	205	,	190	,	112	},
									{	205	,	190	,	112	},					{	205	,	190	,	112	},
									{	205	,	190	,	112	},					{	205	,	190	,	112	},
									{	205	,	190	,	112	},					{	205	,	190	,	112	},
									{	205	,	190	,	112	},					{	194	,	179	,	106	},
									{	183	,	169	,	100	},					{	172	,	159	,	94	},
									{	161	,	149	,	88	},					{	150	,	139	,	82	},
									{	139	,	129	,	76	},					{	139	,	129	,	76	},
									{	139	,	129	,	76	},					{	139	,	129	,	76	},
									{	139	,	129	,	76	},					{	139	,	129	,	76	},
									{	139	,	129	,	76	},					{	139	,	129	,	76	},
									{	139	,	129	,	76	},					{	139	,	129	,	76	},
									{	139	,	129	,	76	},					{	139	,	130	,	83	},
									{	139	,	132	,	91	},					{	139	,	134	,	99	},
									{	139	,	135	,	106	},					{	139	,	137	,	114	},
									{	139	,	139	,	122	},					{	139	,	139	,	122	},
									{	139	,	139	,	122	},					{	139	,	139	,	122	},
									{	139	,	139	,	122	},					{	139	,	139	,	122	},
									{	139	,	139	,	122	},					{	139	,	139	,	122	},
									{	139	,	139	,	122	},					{	139	,	139	,	122	},
									{	139	,	139	,	122	},					{	133	,	133	,	119	},
									{	127	,	127	,	116	},					{	122	,	122	,	113	},
									{	116	,	116	,	110	},					{	110	,	110	,	107	},
									{	105	,	105	,	105	},					{	105	,	105	,	105	},
									{	105	,	105	,	105	},					{	105	,	105	,	105	},
									{	105	,	105	,	105	},					{	105	,	105	,	105	},
									{	105	,	105	,	105	},					{	105	,	105	,	105	},
									{	105	,	105	,	105	},					{	105	,	105	,	105	},
									{	105	,	105	,	105	},					{	119	,	119	,	119	},
									{	133	,	133	,	133	},					{	147	,	147	,	147	},
									{	161	,	161	,	161	},					{	175	,	175	,	175	},
									{	190	,	190	,	190	},					{	190	,	190	,	190	},
									{	190	,	190	,	190	},					{	190	,	190	,	190	},
									{	190	,	190	,	190	},					{	190	,	190	,	190	},
									{	190	,	190	,	190	},					{	190	,	190	,	190	},
									{	190	,	190	,	190	},					{	190	,	190	,	190	},
									{	190	,	190	,	190	},					{	200	,	200	,	200	},
									{	211	,	211	,	211	},					{	222	,	222	,	222	},
									{	233	,	233	,	233	},					{	244	,	244	,	244	},
									{	255	,	255	,	255	},					{	255	,	255	,	255	},
									{	255	,	255	,	255	},					{	255	,	255	,	255	},
									{	255	,	255	,	255	},					{	255	,	255	,	255	},
									{	255	,	255	,	255	},					{	255	,	255	,	255	},
									{	255	,	255	,	255	},					{	255	,	255	,	255	}};


	for (i=0; i<256; i++)
	{
		redlut[i] = terracolors[i].red;
		greenlut[i] = terracolors[i].green;
		bluelut[i] = terracolors[i].blue;
	}
}


/* A colormap from the lookup tables */

rgbtriplet cmap_lut (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;	

	i = (int)(255.0*l);
	rgb = (rgbtriplet){redlut[i], greenlut[i], bluelut[i]};
	return rgb;
}


/* Saturation colormap. Returns blue for underexposed pixels (l<0.02)
	and red for everexposed pixels (l>0.98). All others remain gray.
	Converted to 0-255, the blue pixels are 0-5 and the red pixels 251-255 */

rgbtriplet cmap_saturation (float l, unsigned char iszero)
{

	if (l<0.02) 
		return (rgbtriplet){0,0,255};
	else if (l>0.98) 
		return (rgbtriplet){255,0,0};
	else
		return cmap_grayscale (l,1);
}



/* Two isochromic colormaps in preparation of the plusminus colormap.
	The first runs from green to yellow to whitish-yellow (excluding white),
	and the second runs from blue to magenta to almost-red
*/

rgbtriplet cmap_greenyellow (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = (int)(255.0*l);
	rgb = (rgbtriplet){0,255,0};

	if (i<128) rgb.green = 2*i;
	if (i>=64) rgb.red = MIN(2*(i-64),255);
	if (i>=192) rgb.blue = 3*(i-192);		/* the 3 excludes white */
	return rgb;
}


rgbtriplet cmap_bluered (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = (int)(255.0*l);
	rgb = (rgbtriplet){0,0,0};

//	if (i<128) rgb.blue = 2*i; else if (i>=192) rgb.blue=1023-4*i;
	if (i<128) rgb.blue = 2*i; else rgb.blue=511-2*i;
	if (i>=128) rgb.red = 2*(i-128);

	return (rgb);
}



/* A signed isochromic colormap. Here -1<=l<=1 is allowed */

rgbtriplet cmap_plusminus (float l, unsigned char iszero)
{

	if (l>=0)
		return cmap_greenyellow (l, iszero);
	else
		return cmap_bluered (-l, iszero);

}




/* Two more isochromic colormaps in preparation of the Doppler US colormap.
	The first runs from red to yellow (positive values)
	and the second runs from blue to cyan.
*/

rgbtriplet cmap_redyellow (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = (int)(255.0*l);
	rgb = (rgbtriplet){0,0,0};

	if (i<128)
		rgb.red = 2*i;
	else
	{
		rgb.red = 255;
		rgb.green = 2*(i-128);
	}
	return rgb;
}


rgbtriplet cmap_bluecyan (float l, unsigned char iszero)
{
rgbtriplet rgb;
int i;

	i = (int)(255.0*l);
	rgb = (rgbtriplet){0,0,0};

	if (i<128)
		rgb.blue = 2*i;
	else
	{
		rgb.blue = 255;
		rgb.green = 2*(i-128);
	}
	return (rgb);
}



/* Signed isochromic colormap similar to Doppler ultrasound.
	Here -1<=l<=1 is allowed */

rgbtriplet cmap_dopplerus (float l, unsigned char iszero)
{

	if (l>=0)
		return cmap_redyellow (l, iszero);
	else
		return cmap_bluecyan (-l, iszero);

}





/* The general colormap function, calls the selected individual functions. */


rgbtriplet cmap (float val, float min, float max)
{
float f;
unsigned char iszero;

	if ((active_cmap==9) || (active_cmap==10))			/* Signed isochromic colormaps */
	{
		if ((val<0) && (min<0))
			f = -(fabs(val)/fabs(min));
		else if (max>0)
			f = val/max;
		else
			f=0;
		if (active_cmap==9)
			return cmap_plusminus (bcg(f), 1);
		else
			return cmap_dopplerus (bcg(f), 1);
	}


	iszero = F_ISZERO(val); f=0.0;
	if (F_ISZERO(max-min))
		return cmap_grayscale (0.0,1);		/* zero image value range returns all-black */
	else
		f = (val-min)/(max-min);			/* Scale into 0...1 range */

	if (f<0.0) f=0.0; else if (f>1.0) f=1.0;	/* Clamp in case somebody messed with the "trim image" function */

	if ((!active_cmap) || (!cmap_function))		/* If no color map exit with grayscale */
		return cmap_grayscale (bcg(f), iszero);

	return cmap_function (bcg(f), iszero);

}


/* To apply log scale to RGB images. Each color is logarithmized (value range now 0...2.4)
	and rescaled with a multiplication by 100. This goes via float.
	Since rgb is unsigned, no difference between log and logabs exists. */

rgbtriplet rgb2log (rgbtriplet rgb)
{
float f;
int i;
rgbtriplet result;

	f = SAFELOG10((float)(rgb.red)+1.0);		/* Offset by 1 to avoid log(0) */
	i = IROUND (f*105.0);
	if (i<0) i=0; else if (i>255) i=255;
	result.red = i;

	f = SAFELOG10((float)(rgb.green)+1.0);
	i = IROUND (f*105.0);
	if (i<0) i=0; else if (i>255) i=255;
	result.green = i;

	f = SAFELOG10((float)(rgb.blue)+1.0);		/* Offset by 1 to avoid log(0) */
	i = IROUND (f*105.0);
	if (i<0) i=0; else if (i>255) i=255;
	result.blue = i;

	return result;
}




/********************************************************

	This function copies the main_img buffer contents into
	the imlib buffer for display. It also converts any
	main_img type into the exclusively used RGB type
	of the imlib buffer.

***********************************************************/



void make_imlib_buffer_from_main_image (int slice)
{
rgbtriplet* p1; unsigned char *temp;
int x,y, tx, ty; float fx, fy, xfac, yfac;
float min, max, buf;
long i,z, xy;
rgbtriplet rgb, rgb2, rgb3;
interpol_method intp;

	/* Generate an RGB data buffer that can be used to create the pixbuf from.
		The data buffer is used directly in the pixbuf. Any function that 
		calls this function needs to call "redraw", i.e., emit an
		expose event. */

	tx = mainimg.xmax;	xfac=1.0;
	ty = mainimg.ymax;	yfac=1.0;

	/* Scale tx, ty by zoomlevel. Use a loop to avoid a floating-point operation */

	if (zoomlevel_x>0)			/* Zoom in (enlarge) */
	{
		for (i=0; i<zoomlevel_x; i++)
		{
			tx <<= 1; xfac /=2.0;
		}
	}
	else if (zoomlevel_x<0)		/* Zoom out (shrink) */
	{
		for (i=0; i< (-zoomlevel_x); i++)
		{
			tx >>= 1; xfac *=2.0;
		}
	}

	if (zoomlevel_y>0)			/* Zoom in (enlarge) */
	{
		for (i=0; i<zoomlevel_y; i++)
		{
			ty <<= 1; yfac /=2.0;
		}
	}
	else if (zoomlevel_y<0)		/* Zoom out (shrink) */
	{
		for (i=0; i< (-zoomlevel_y); i++)
		{
			ty >>= 1; yfac *=2.0;
		}
	}

	/* Let's decide which is the best interpolation method. If we shrink, NN is best,
		because we always land on integer coordinates and don't need to interpolate.
		If we zoom in, BILIN might just do. No need for fancier interpolation here. */

	if ((xfac<1.0) || (yfac<1.0))			/* We enlarge */
	{
		if (intp_pref==INTP_NN)
			intp = INTP_NN;					/* No need to be fancier than global */
		else
			intp = INTP_BILIN;
	}
	else									/* We shrink */
		intp = INTP_NN;

	temp = (unsigned char*) calloc (tx*ty, sizeof (rgbtriplet));
	if (!temp) fatal_memerr();
	p1 = (rgbtriplet *)temp;

	/* Compute index into current slice (of main image) */

	if (slice>=mainimg.zmax) slice=mainimg.zmax-1; else if (slice<0) slice=0;
	xy = mainimg.xmax*mainimg.ymax;
	z = xy*slice;
	imgminmax (&mainimg, &min, &max);
	if (cmap_use_logviz)
	{
		if ((mainimg.imgtype == B_CHAR) || (mainimg.imgtype==B_RGB))
		{
			min=0; max=SAFELOG10(max+1);		/* add 1 to avoid log(0) */
		}
		else
		{
			if (cmap_use_logviz==1) 
				min = SAFELOG10(min);		/* Only when using pure log */
			else
				min = 0;					/* Default when using log-abs */
			max = SAFELOG10(max);
		}
	}
	
	switch (mainimg.imgtype)
	{
		case B_CHAR:
		case B_SHORT:
		case B_FLOAT:
		{
			for (y=0, fy=0.0; y<ty; y++, fy+=yfac)
				for (x=0, fx=0.0; x<tx; x++, fx+=xfac)		/* Loop over pixels of target buffer */
				{
					buf = ireadbuf (&mainimg, fx, fy, slice, intp);

					if (cmap_use_logviz && (mainimg.imgtype==B_CHAR)) buf+=1;	/* add 1 to avoid log(0) */
					if (cmap_use_logviz==2) 
						buf = SAFELOG10 (fabs(buf));
					else if (cmap_use_logviz==1) 
						buf = SAFELOG10 (buf);

					rgb = cmap (buf, min, max);
					*p1++ = rgb;
				}
			break;
		}
		case B_RGB:			/* Let's try to be funny with RGB: Shove each color through the colormaps ... */
		{					/* ... and grab only the corresponding component. */
			for (y=0, fy=0.0; y<ty; y++, fy+=yfac)
				for (x=0, fx=0.0; x<tx; x++, fx+=xfac)		/* Loop over pixels of target buffer */
				{
					rgb = rgbireadbuf (&mainimg, fx, fy, slice, intp);
					if (cmap_use_logviz)
						rgb = rgb2log (rgb);			/* No difference between log and logabs */
					rgb2 = cmap (rgb.red, min, max);
					rgb3.red=rgb2.red;
					rgb2 = cmap (rgb.green, min, max);
					rgb3.green=rgb2.green;
					rgb2 = cmap (rgb.blue, min, max);
					rgb3.blue=rgb2.blue;
					*p1++ = rgb3;
				}
			break;
		}
	}

	/* If requested, add contour lines to image space in p1 */

	p1 = (rgbtriplet *)temp;
	if (cmap_use_contours)
		make_contour_lines (&mainimg, p1, ncontours, slice, xfac, yfac, tx, (rgbtriplet){0,0,0});


	/* Now use the temp buffer to create the final pixbuf. 
	    Also, initialize misc variables, and adjust the scrollbars */

	if (mainimg.pixbuf)  g_object_unref (G_OBJECT(mainimg.pixbuf));

	mainimg.pixbuf = gdk_pixbuf_new_from_data (temp, GDK_COLORSPACE_RGB, FALSE, 8, 
														tx, ty, 3*tx 		/* rowstride */,
														pixbuf_destroy_handler		 /* destroy callback */,
														&mainimg.pixbuf				 /* destroy callback user_data */);

	/* Check if a size change is needed, activate the appropriate callback */

	if ((tx!=mainimg.width) || (ty!=mainimg.height))
		mainwindow_resize_callback (&mainimg, tx, ty);

	/* and discard the temp buffer */
	//free (temp);			NOTE! Must not discard this buffer, because the memory is used in the pixbuf.
	
	/* adjust the z spinbutton in case something changed and remember the slice we are looking at */
	gtk_adjustment_init_val ((GtkAdjustment*)slice_spinbutton_adj, slice, 0, mainimg.zmax-1);
	cur_slice = slice;

}


/* Almost verbatim copy of the above. It performs the false-coloring and adds contour lines
	to a NEW image which is returned in img. This function DOES NOT consider the zoom
	level. The source image is generalized as src. */

void create_viz_from_main_image (image_type *src, image_type *img, int slice)
{
rgbtriplet* p1;
long i,z, xy;
rgbtriplet rgb, rgb2, rgb3;
float buf,min,max;

	if (allocate_image (img, B_RGB, src->xmax, src->ymax, 1)==-1)
		fatal_memerr();
	p1=img->data;

	imgminmax (&mainimg, &min, &max);
	if (cmap_use_logviz)
	{
		if ((mainimg.imgtype == B_CHAR) || (mainimg.imgtype==B_RGB))
		{
			min=0; max=SAFELOG10(max+1);		/* add 1 to avoid log(0) */
		}
		else
		{
			if (cmap_use_logviz==1) 
				min = SAFELOG10(min);		/* Only when using pure log */
			else
				min = 0;					/* Default when using log-abs */
			max = SAFELOG10(max);
		}
	}

	/* Generate a RGB data buffer that can be used to create an imlib image from data 
	   The reason for using this somewhat convoluted method is the lack of a
	   "create blank image" function in imlib. */

	if (slice>=src->zmax) slice=src->zmax-1; else if (slice<0) slice=0;
	xy = src->xmax*src->ymax;
	z = xy*slice;

	switch (src->imgtype)
	{
		case B_CHAR:
		{
			unsigned char *p2 = src->data;
			for (i=0; i<xy; i++)
			{
				buf = p2[z+i]; 
				if (cmap_use_logviz)  buf = SAFELOG10 (buf+1);			/* Special case: Nonchalantly offset by 1 to avoid log(0) */
				rgb = cmap (p2[z+i], min, max);
				*p1++ = rgb;
			}
			break;
		}
		case B_SHORT:
		{
			short *p2 = src->data;
			for (i=0; i<xy; i++)
			{
				buf = p2[z+i]; 
				if (cmap_use_logviz==2) 
					buf = SAFELOG10 (fabs(buf));
				else if (cmap_use_logviz==1) 
					buf = SAFELOG10 (buf);
				rgb = cmap (buf, min, max);
				*p1++ = rgb;
			}
			break;
		}
		case B_RGB:			/* NOTE that RGB does not allow the use of color maps */
		{
			rgbtriplet *p2 = src->data;
			for (i=0; i<xy; i++)
			{
				rgb = p2[z+i];
				if (cmap_use_logviz)
						rgb = rgb2log (rgb);			/* No difference between log and logabs */
				rgb2 = cmap (rgb.red, min, max);
				rgb3.red=rgb2.red;
				rgb2 = cmap (rgb.green, min, max);
				rgb3.green=rgb2.green;
				rgb2 = cmap (rgb.blue, min, max);
				rgb3.blue=rgb2.blue;
				*p1++ = rgb3;
			}
			break;
		}
		case B_FLOAT:
		{
			float* p2 = src->data;

			for (i=0; i<xy; i++)
			{
				buf = p2[z+i]; 
				if (cmap_use_logviz==2) 
					buf = SAFELOG10 (fabs(buf));
				else if (cmap_use_logviz==1) 
					buf = SAFELOG10 (buf);
				rgb = cmap (buf, min, max);
				*p1++ = rgb;
			}
			break;
		}
	}

	/* If requested, add contour lines to image space in img->data. DO NOT use p1 any longer */

	if (cmap_use_contours)
		make_contour_lines (src, img->data, ncontours, slice, 1.0, 1.0, img->xmax, (rgbtriplet){0,0,0});

	img->minval.imin=0; img->maxval.imax=255;
}




/************************************************************************************************************/




/* Create a MIP projection of a stack */

void secondary_mip (image_type* img)
{
int x,y,z;
rgbtriplet mip;
rgbtriplet* tempimg;
long idx, zofs;
unsigned char d;


	close_secondary_image ();

	/* Allocate a bitmap to take normalized MIP data */
	
	tempimg = (rgbtriplet*) calloc (img->xmax * img->ymax, sizeof (rgbtriplet));
	if (!tempimg) fatal_memerr ();

	/* TODO: Do we get a memory leak if alternating between MIP and secondary display ? */

	if (allocate_image (&s_img, img->imgtype, img->xmax, img->ymax, 1)==-1)
		fatal_memerr ();


	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
		
			idx = x + y*img->xmax;
			zofs = img->xmax * img->ymax;
			switch (img->imgtype)
			{
				case B_CHAR:
				{
					unsigned char* p = img->data;
					unsigned char* p1 = s_img.data;
					unsigned char max;
					p += idx;
					max = mip.red = *p;
					if (img->zmax>0)
						for (z=1; z<img->zmax; z++)
						{
							p+= zofs;
							d = IROUND(255.0*( (float)(*p - img->minval.imin) 
										/ (float) (img->maxval.imax - img->minval.imin)));
							if ( d > mip.red) mip.red = d;
							if (max < *p) max = *p;
						}
					p1[idx] = mip.red;
					mip.green = mip.blue = mip.red;
					tempimg[idx]=mip;
					p1[idx] = max;
					break;
				}
				case B_SHORT:
				{
					short* p = img->data;
					short* p1 = s_img.data;
					short max;
					p += idx;
					max = *p;
					mip.red = IROUND(255.0*( (float)(*p - img->minval.imin) 
								/ (float) (img->maxval.imax - img->minval.imin)));
					if (img->zmax>0)
						for (z=1; z<img->zmax; z++)
						{
							p+= zofs; 
							d = IROUND(255.0*( (float)(*p - img->minval.imin) 
										/ (float) (img->maxval.imax - img->minval.imin)));
							if ( d > mip.red) mip.red = d;
							if (max < *p) max = *p;
						}
					mip.green = mip.blue = mip.red;
					tempimg[idx]=mip;
					p1[idx] = max;
					break;
				}
				case B_FLOAT:
				{
					float* p = img->data;
					float* p1 = s_img.data;
					float max;
					p += idx;
					max = *p;
					mip.red = IROUND(255.0*( (float)(*p - img->minval.fmin) 
								/ (float) (img->maxval.fmax - img->minval.fmin)));
					if (img->zmax>0)
						for (z=1; z<img->zmax; z++)
						{
							p+= zofs;
							d = IROUND(255.0*( (float)(*p - img->minval.fmin) 
										/ (float) (img->maxval.fmax - img->minval.fmin)));
							if ( d > mip.red) mip.red = d;
							if (max < *p) max = *p;
						}
					mip.green = mip.blue = mip.red;
					tempimg[idx]=mip;
					p1[idx] = max;
					break;
				}
				case B_RGB:			/* Note: No contrast rescaling with RGB! */
				{
					rgbtriplet* p = img->data;
					rgbtriplet* p1 = s_img.data;
					p += idx;
					mip.red   = p->red;
					mip.green = p->green;
					mip.blue  = p->blue;
					if (img->zmax>0)
						for (z=1; z<img->zmax; z++)
						{
							p+= zofs;
							if ( p->red   > mip.red)   mip.red   = p->red;
							if ( p->green > mip.green) mip.green = p->green;
							if ( p->blue  > mip.blue)  mip.blue  = p->blue;
						}
					tempimg[idx]=p1[idx]=mip;
					break;
				}
			}
		}

	make_minmax (&s_img);

	/* Now create an imlib image from the temp data */
	
	/* Now create a pixbuf from the temp data */
	
	if (s_img.pixbuf) gtk_object_unref (GTK_OBJECT(s_img.pixbuf));

	s_img.pixbuf = gdk_pixbuf_new_from_data ((unsigned char*)tempimg, GDK_COLORSPACE_RGB, FALSE, 8, 
														s_img.xmax, s_img.ymax, 3*s_img.xmax /* rowstride */,
														pixbuf_destroy_handler /* destroy callback */,
														s_img.pixbuf /* destroy callback user_data */);

    secw=s_img.xmax; sech=s_img.ymax;
	
	/* Display the newly created image */
				
	secondaryimage_exists = 1;
	create_secondwnd (&s_img, "Maximum Intensity Projection through Z");
	
	gdk_flush();
/*	gdk_imlib_destroy_image (im); */
		

}




/******************************************************************

	view_zslab cuts the image in z-direction along the two
	most recently clicked points and displays this slab
	in the secondary display.
	The first part of this function is similar to the
	profile display function, but we do not allow different
	z-coords.
	This function always uses bilinear interpolation, because
	the spline interpolation is inacceptably slow (continuous change of Z)

*********************************************************************/

/* TODO: We should be able to interpolate in Z to vary the aspect ratio */


void view_zslab ()
{
vertex3d start, end;
double x,y;
int z;
double dx,dy;
int numpts,i;
float* p1;
rgbtriplet* p2;
int e;
image_type slab;

	/* Get the two most recent points that have been clicked */

	if (clickstack_last < 2) return;
	if (mainimg.zmax < 2) return;
	start = clickstack[clickstack_last-2];
	end   = clickstack[clickstack_last-1];
	if ((start.x==end.x) && (start.y==end.y)) return;
	e = 1; if (mainimg.imgtype==B_RGB) e=3;

	dx = end.x - start.x;
	dy = end.y - start.y;
	
	if (fabs(dx)>fabs(dy))					/* x-dominant */
		numpts = abs(end.x - start.x);
	else 									/* y-dominant */
		numpts = abs(end.y - start.y);

	dx /= numpts;
	dy /= numpts;

	/* Now allocate a new image of numpoints x zmax x 1 pixels */
	
	
	if (mainimg.imgtype==B_RGB)
		i = allocate_image (&slab, B_RGB, numpts, mainimg.zmax, 1);
	else
		i = allocate_image (&slab, B_FLOAT, numpts, mainimg.zmax, 1);
		
	if (i==-1) fatal_memerr();
	p1 = slab.data; p2 = slab.data;
	
	/* Copy the image's pixels into the slab */
	
	x = start.x; y = start.y;
	for (i=0; i<numpts; i++)
	{
		for (z=0; z<mainimg.zmax; z++)
		{
			if (mainimg.imgtype==B_RGB)
				p2[i + numpts*z] = rgbireadbuf (&mainimg, x,y,z,INTP_BILIN);
			else
				p1[i + numpts*z] = ireadbuf (&mainimg, x,y,z,INTP_BILIN);
		}
		x+=dx; y+=dy;
	}
	make_minmax (&slab);
	
	/* Display the newly generated image and the line along we cut */
	
	display_clickstack=2;
	secondary_display (&slab, 0, "Z section");
	
	/* Discard the original image as it has been copied */
	
	freebuf (&slab);
	
}



/************************************************************************************************************/

/* This is exactly the problem we run into from the present LUT concept.
	There is no way to integrate a LUT with contour lines in a proper manner.
	Our only way out of this mess (short of doing the hard work and integrating
	the LUT functions here) is to create contour lines in the secondary window
	so they can be saved and merged through image math (*sorry*)
*/


/*    ###     ###    #   #   #####    ###    #   #   ####            #        ###    #   #   #####    ###           */
/*   #   #   #   #   #   #     #     #   #   #   #   #   #           #         #     #   #   #       #   #          */
/*   #       #   #   ##  #     #     #   #   #   #   #   #           #         #     ##  #   #       #              */
/*   #       #   #   # # #     #     #   #   #   #   ####            #         #     # # #   ####     ###           */
/*   #       #   #   #  ##     #     #   #   #   #   # #             #         #     #  ##   #           #          */
/*   #   #   #   #   #   #     #     #   #   #   #   #  #            #         #     #   #   #       #   #          */
/*    ###     ###    #   #     #      ###     ###    #   #           #####    ###    #   #   #####    ###           */


/* The concept of contour lines: We must assume that the exact contour line
	runs between pixels. If we have a contour level C, then each edge
	(connection between two 4-connected pixels) intersects the contour line
	if one pixel is above C and the other below or equal C.
	We can scan the image in inverted-L fashion. We get four cases
	where a contour line intersects (O is below or equal to C; X is above C)

	X----O   	and   	O----X

	X			and		O
	|					|
	O					X

	In each of the four cases, we mark the X pixel as contour line.

	This function is the core of the algorithm. It expects the input image 'img'
	and operates on slice 'slice'. It also expects a pointer to RGB data, 'target'
	which must have the same x,y dimensions as img. Contour lines are set to 'color',
	all other pixels of 'target' remain untouched.
	'numcontours' determines the number of contourlines between image min and max values.
	
	A tricky business with the variable zoom factor. We really need to use the scaled and
	interpolated image buffer to account for the zoom factor. Scalex and scaley are 2 to the power
	of zoom_x and zoom_y. Its reciprocal are the step widths. as a safeguard, the target buffer's
	rowstride is provided explicitly.
	
*/


void make_contour_lines (image_type *img, rgbtriplet *target, int numcontours, int slice, float scalex, float scaley, int rowstride, rgbtriplet color)
{
int x,y,i,idx, idx1;
float x1,y1, dx, dy;
float min, max, buf, bx, by;
float *C;					/* The contour levels */


	imgminmax (img, &min, &max);		/* To determine the contour levels */
	C = calloc (numcontours+5, sizeof (float));
	for (i=0; i<numcontours; i++)
		C[i] = min + (i+1)*(max-min)/(numcontours+1);		/* Linear contour levels between min & max */

	dx = scalex; dy = scaley;

	for (y=0, y1=0.0; y1<img->ymax-1; y1+=dy, y++)
	{
		idx = y*rowstride; idx1 = idx+rowstride;
		for (x=0, x1=0.0; x1<img->xmax-1; x1+=dx, x++)
		{
			buf = ireadbuf (img, x1,y1,slice, intp_pref);
			bx  = ireadbuf (img, x1+dx,y1,slice, intp_pref);
			by  = ireadbuf (img, x1,y1+dy,slice, intp_pref);
			for (i=0; i<numcontours; i++)		/* For each contour level...  */
			{
				if ((buf>C[i]) && (bx<=C[i])) target[idx] = color;
				else if ((buf>C[i]) && (by<=C[i])) target[idx] = color;
				if ((buf<=C[i]) && (bx>C[i])) target[idx+1] = color;
				if ((buf<=C[i]) && (by>C[i])) target[idx1] = color;
			}
			idx++; idx1++;
		}
	}

	free (C);

}



/* The GUI driver, to be extended (variable contour lines, variable distance, power-law contour lines... */

void show_contour_lines (image_type *img, int slice)
{
image_type cont;

	if (allocate_image (&cont, B_RGB, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();

	make_contour_lines (img, cont.data, ncontours, slice, 1.0, 1.0, cont.xmax, (rgbtriplet){255,255,255});
	cont.maxval.imax=255; cont.minval.imin=0;
	secondary_display (&cont, 0, "Contour lines");

}




/*******************************************************************************************************************/

/*          ###     ###    #        ###    ####    #   #     #     ####             ###    #   #    ###           */
/*         #   #   #   #   #       #   #   #   #   ## ##    # #    #   #           #   #   #   #     #            */
/*         #       #   #   #       #   #   #   #   # # #   #   #   #   #           #       #   #     #            */
/*         #       #   #   #       #   #   ####    # # #   #   #   ####            #  ##   #   #     #            */
/*         #       #   #   #       #   #   # #     #   #   #####   #               #   #   #   #     #            */
/*         #   #   #   #   #       #   #   #  #    #   #   #   #   #               #   #   #   #     #            */
/*          ###     ###    #####    ###    #   #   #   #   #   #   #                ###     ###     ###           */
/*                                                                                                                */


/* The color map GUI. Lets the user select several canned color maps or a
	user-defined color map from RGB curves. We don't actually do anything here
	as the colormap is applied in make_imlib_buffer_from_main_image
*/

/* Unfortunately, the GtkCurve widget has been discontinued after Gtk-1,2.
	The widget exists in Gtk-2.x, but is incredibly buggy, so we can't
	use it any more. Instead, let' use a general drawingarea for
	display, but no longer manipluation, of the curves. */


GtkWidget *cmap_bccurve;			/* Drawingarea for the brightness-contrast-gamma curve */
GtkWidget *cmap_lutcurve;			/* Drawingarea for the colormap curves */
GtkWidget *cmap_window;
GtkWidget *predef_cmap;
GtkWidget *cmap_logviz_menu;
GtkWidget *cmap_contour_button;
GtkWidget *cmap_logviz_button;
GtkAdjustment* bc_bright_adj;
GtkAdjustment* bc_contr_adj;
GtkAdjustment* bc_gamma_adj;
GtkAdjustment *cmap_ncontour_adj;
unsigned char vredlut[256], vgreenlut[256], vbluelut[256];		/* For display purposes */


void cmap_get_lut_from_curve ()		/* DEPRECATED with Gtk-2.x and the abandoned GtkCurve widget */
{
int i;
float fredlut[256], fgreenlut[256], fbluelut[256];

	/* Get the LUTs from the curve objects */
	

}


void cmap_transfer_lut ()	/* Transfer current colormap into graph */
{
int i;
rgbtriplet rgb;
float bright, contr, gamma;					/* To save the original values */

	bright = viz_bright; contr = viz_contrast; gamma=viz_gamma;
	viz_bright = 0; viz_contrast = 1; viz_gamma=1;

	for (i=0; i<256; i++)
	{
		rgb = cmap (i,0,255);
		vredlut[i]= rgb.red;
		vgreenlut[i]= rgb.green;
		vbluelut[i]= rgb.blue;
	}

	viz_bright = bright; viz_contrast = contr; viz_gamma=gamma;


}


void curve_redraw (unsigned char which)
{
GdkGC *gc, *gc1, *gc2;
GdkColor color;
int cw, ch, lw, lh;
int i,j,c,x1,y1,x2,y2;
rgbtriplet rgb;

	dp (2, "Colormap: curve redraw (curve %d)\n", which);
	gc = gdk_gc_new (cmap_bccurve->window);	/* allocate mem & assign null values */
	gdk_gc_copy (gc, cmap_bccurve->style->white_gc); /* Copy some values */
	cw = cmap_bccurve->allocation.width;
	ch = cmap_bccurve->allocation.height;
	lw = cmap_lutcurve->allocation.width;
	lh = cmap_lutcurve->allocation.height;

	if (which & 1)
		gdk_draw_rectangle (cmap_bccurve->window, gc, TRUE, 0,0,cw,ch);

	if (which & 2)
		gdk_draw_rectangle (cmap_lutcurve->window, gc, TRUE, 0,0,lw,lh);

	/* After clearing the drawingareas, switch to black */

	gdk_gc_copy (gc, cmap_bccurve->style->black_gc); /* Copy some values */
	gc1 = gdk_gc_new (cmap_bccurve->window);	/* allocate mem & assign null values */
	gdk_gc_copy (gc1, cmap_bccurve->style->black_gc); /* Copy some values */
	//gdk_gc_set_dashes (gc1, 0, "\3\3", 2);		/* black dashed */
	//gdk_gc_set_function (gc1, GDK_XOR);
	for (i=64; i<256; i+=64)
	{
		if (which & 1)
		{
			x1 = 0; x2=cw; y1 = y2 = (int)(0.5+ch*(float)i/256.0);
			gdk_draw_line (cmap_bccurve->window, gc1, x1,y1, x2,y2);
			y1 = 0; y2=ch; x1 = x2 = (int)(0.5+cw*(float)i/256.0);
			gdk_draw_line (cmap_bccurve->window, gc1, x1,y1, x2,y2);
		}
		if (which & 2)
		{
			x1 = 0; x2=lw; y1 = y2 = (int)(0.5+lh*(float)i/256.0);
			gdk_draw_line (cmap_lutcurve->window, gc1, x1,y1, x2,y2);
			y1 = 0; y2=lh; x1 = x2 = (int)(0.5+lw*(float)i/256.0);
			gdk_draw_line (cmap_lutcurve->window, gc1, x1,y1, x2,y2);
		}
	}

	if (which & 1)
	{
		x1=0; y1 = ch-(int)(256*bcg(0))-1;
		for (i=1; i<256; i++)
		{
			y2= ch-(int)(ch*bcg((float)i/256.0))-1;
			if (y2<0) y2=0; if (y2>=ch) y2=ch;
			x2 = (int)(cw*i/256.0);
			gdk_draw_line (cmap_bccurve->window, gc1, x1,y1, x2,y2);
			x1=x2; y1=y2;
		}
	}

	if ((which & 2) && (cmap_function))
	{
		gc2 = gdk_gc_new (cmap_lutcurve->window);		/* allocate mem & assign null values */

		gdk_gc_copy (gc2, cmap_lutcurve->style->black_gc); /* Copy some values */
		gdk_gc_set_function (gc2, GDK_AND);

		for (j=0; j<3; j++)
		{
			color.red = (j==0? 65535: 0);
			color.green = (j==1? 65535: 0);
			color.blue = (j==2? 65535: 0);
			gdk_color_alloc (gdk_colormap_get_system(), &color);
			gdk_gc_set_foreground (gc2, &color);
			
			rgb = cmap_function (0.0, 0);
			if (j==0) c=rgb.red; if (j==1) c=rgb.green; if (j==2) c=rgb.blue;

			x1=0; y1 = lh-(int)(256*c)-1;
			for (i=1; i<256; i++)
			{
				rgb = cmap_function (i/256.0, (i==128));
				if (j==0) c=rgb.red; if (j==1) c=rgb.green; if (j==2) c=rgb.blue;
				y2= lh-(int)(lh*(float)c/256.0)-1;
				if (y2<0) y2=0; if (y2>=ch) y2=ch;
				x2 = (int)(lw*i/256.0);
				gdk_draw_line (cmap_lutcurve->window, gc2, x1,y1, x2,y2);
				x1=x2; y1=y2;
			}
		}

		gdk_gc_destroy (gc2);
	}


	gdk_gc_destroy (gc);	/* free the memory associated with the gc */
	gdk_gc_destroy (gc1);

}


void cmap_go (int prep_only)
{

	switch (active_cmap)
	{
		case 0:
		case 1:
			cmap_function = NULL;
			break;
		case 2:
			cmap_function = &cmap_negative;
			break;
		case 3:
			cmap_function = &cmap_rgb_simple;
			break;
		case 4:
			cmap_function = &cmap_rgbk_simple;
			break;
		case 5:
			cmap_function = &cmap_fire;
			break;
		case 6:
			cmap_function = &cmap_rainbow;
			break;
		case 7:
			cmap_function = &cmap_rainbow_k;
			break;
		case 8:
			cmap_terra_prep();						/* Copy discrete LUT into tables*/
			cmap_function = &cmap_lut;
			break;
		case 9:
		case 10:
			cmap_function = &cmap_grayscale;		/* Special cases trapped in cmap */
			break;
		case 11:
			cmap_function = &cmap_saturation;
			break;
		case 12:
			cmap_get_lut_from_curve ();
			cmap_function = &cmap_lut;
			break;
		default:
			cmap_function = NULL;
	}
	
	if (!prep_only) 
	{
		cmap_transfer_lut ();
		curve_redraw (2);
		make_imlib_buffer_from_main_image (cur_slice);
		redraw (0);
	}

}



void on_cmap_reset_clicked (GtkButton *button, gpointer user_data)
{

	dp (2, "Colormap: curve reset\n");

	init_cmap_lut ();
	viz_bright = 0.0; viz_contrast = 1.0; viz_gamma = 1.0;
	gtk_adjustment_set_value (bc_bright_adj,0.0);
	gtk_adjustment_set_value (bc_contr_adj,0.0);
	gtk_adjustment_set_value (bc_gamma_adj,1.0);

	active_cmap = 0; cmap_function = NULL;
	cmap_use_contours=0;
	cmap_use_logviz = 0;
	gtk_option_menu_set_history (GTK_OPTION_MENU(predef_cmap), active_cmap);
	gtk_option_menu_set_history (GTK_OPTION_MENU(cmap_logviz_menu), cmap_use_logviz);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cmap_contour_button), cmap_use_contours);
	cmap_transfer_lut();
	make_imlib_buffer_from_main_image (cur_slice);
	redraw (0);
	if (stat_window_exists)
		statop_redraw_all (0);			/* Redraw the color bar in the statop window if it exists */

}


/* Configure and expose events for the brightness-contrast curve */

gboolean on_cmap_bcconfigure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
	curve_redraw (1);
}


gboolean on_cmap_bcexpose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	curve_redraw (1);
}

gboolean on_cmap_lutconfigure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
	curve_redraw (2);
}


gboolean on_cmap_lutexpose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	curve_redraw (2);
}



void on_cmap_create_clicked (GtkButton *button, gpointer user_data)
{
image_type img;

	dp (2, "Colormap: Create\n");

	cmap_go (1);
	create_viz_from_main_image (&mainimg, &img, cur_slice);
	secondary_display (&img,0,"Image with LUT");
	freebuf (&img);
}


void on_cmap_ok_clicked (GtkButton *button, gpointer user_data)
{

	dp (2, "Colormap: curve apply\n");
	cmap_go (0);
	if (stat_window_exists)
		statop_redraw_all (0);			/* Redraw the color bar in the statop window if it exists */

}



void bc_go (GtkWidget* widget, gpointer user_data)
{
float bright, contrast, gamma;


	bright = bc_bright_adj->value;
	contrast = bc_contr_adj->value + 45.0;		/* tan 45 = 1.0 */
	viz_gamma = 1.0/bc_gamma_adj->value;
	viz_contrast = tan(contrast*PI/180.0);
	viz_bright = bright/127.0;

	dp (2, "Brightness=%5.1f    Contrast=%5.1f     Gamma=%3.1f\n", viz_bright, viz_contrast, viz_gamma);
	curve_redraw (1);
	make_imlib_buffer_from_main_image (cur_slice);
	redraw (0);
	//	cmap_go (0);

}


void on_cmap_cancel_clicked (GtkButton *widget, gpointer user_data)
{

	cmap_window_open = 0;
	gtk_widget_destroy (GTK_WIDGET(user_data));
}


void cmap_curvemenu_handler (GtkWidget *widget, gpointer user_data) /* DEPRECATED */
{
int i;
GtkCurveType type;

	i = *(int*) user_data;
	

}


void cmap_optionmenu_handler (GtkWidget *widget, gpointer user_data)
{
int i;

	i = *(int*) user_data;
	active_cmap = i;

}


void cmap_logviz_handler (GtkWidget *widget, gpointer user_data)
{
int i;

	cmap_use_logviz = *(int*) user_data;

	if (cmap_use_logviz==2)
		dp (2, "Colormap: Now displaying log-abs data\n");
	else if (cmap_use_logviz==1)
		dp (2, "Colormap: Now displaying log data\n");
	else
		dp (2, "Colormap: Turned off log, now displaying linear data\n");

}


void on_cmap_contourbutton_clicked (GtkWidget *widget, gpointer user_data)
{

 	cmap_use_contours = (*(GtkToggleButton*)widget).active;
	dp (2, "Colormap: contour lines set to %d\n",cmap_use_contours);

}

void cmap_ncontour_changed (GtkWidget* widget, gpointer user_data)
{
	ncontours = cmap_ncontour_adj->value;
	dp (2, "Colormap: Number of contour lines set to %d\n",ncontours);
}




void create_cmap_window (void)
{
  GtkWidget *vbox1, *table;
  GtkWidget *cmap_label1, *label2, *label3, *label4, *label5, *label6, *label7;
  GtkWidget *hbox1, *hbox2, *hbox3, *hbox4;
  GtkWidget *cmap_type;
  GtkWidget *cmap_type_menu, *predef_cmap_menu, *cmap_logviz_menu_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *glade_menuitem1;
  GtkWidget *hsep, *vsep;
  GtkWidget *cmap_ok;
  GtkWidget *cmap_preview;
  GtkWidget *cmap_reset;
  GtkWidget *cmap_cancel;
  GtkWidget *cmap_create;
  GtkWidget *bc_bright, *bc_contr, *bc_gamma, *cmap_ncontour;
  int i;

	for (i=0; i<256; i++)
	{
		vredlut[i]=vgreenlut[i]=vbluelut[i]=i;
	}


	cmap_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (cmap_window), "cmap_window", cmap_window);
	gtk_window_set_title (GTK_WINDOW (cmap_window), "Color Map Selection");

	vbox1 = vbox_in_window (cmap_window,"vbox1");
	table = MakeTable (cmap_window, vbox1, "table1", 3,2);

	/* Brightness / Contrast / Gamma sliders */

	label2 = MakeLabel (cmap_window,table,"Brightness","cmap_label2",0,0);
	label3 = MakeLabel (cmap_window,table,"Contrast","cmap_label3",1,0);
	label4 = MakeLabel (cmap_window,table,"Gamma","cmap_label4",2,0);
	bc_bright_adj = GTK_ADJUSTMENT (gtk_adjustment_new (0, -127, 127, 1, 10, 0));
	bc_bright = MakeHScale (bc_bright_adj,cmap_window,table,"bc_bright",0,1);

	bc_contr_adj = GTK_ADJUSTMENT (gtk_adjustment_new (0, -45, 44, 1, 10, 0));
	bc_contr = MakeHScale (bc_contr_adj,cmap_window,table,"bc_contr",1,1);

	bc_gamma_adj = GTK_ADJUSTMENT (gtk_adjustment_new (1, 1, 3, 1, 1, 0));
	bc_gamma = MakeHScale (bc_gamma_adj,cmap_window,table,"bc_gamma",2,1);

	hsep = MakeHsep (cmap_window,vbox1,"hseparator2",-1,-1);

	hbox4 = gtk_hbox_new (TRUE, 0);
	gtk_widget_ref (hbox4);
	gtk_object_set_data_full (GTK_OBJECT (cmap_window), "hbox4", hbox4,
                              (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox4);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox4, TRUE, TRUE, 10);

	label2 = MakeLabel (cmap_window,hbox4,"Grayscale Remapping","cmap_label2",-1,-1);
	label3 = MakeLabel (cmap_window,hbox4,"Color Curves","cmap_label3",-1,-1);

	/* Curve graphs */

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (cmap_window), "hbox1", hbox1,
                              (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

	cmap_bccurve = gtk_drawing_area_new ();
	gtk_widget_ref (cmap_bccurve);
	gtk_widget_show (cmap_bccurve);
	gtk_box_pack_start (GTK_BOX (hbox1), cmap_bccurve, TRUE, TRUE, 5);
	gtk_widget_set_size_request (GTK_WIDGET (cmap_bccurve), 256, 256);

	cmap_lutcurve = gtk_drawing_area_new ();
	gtk_widget_ref (cmap_lutcurve);
	gtk_widget_show (cmap_lutcurve);
	gtk_box_pack_start (GTK_BOX (hbox1), cmap_lutcurve, TRUE, TRUE, 5);
	gtk_widget_set_size_request (GTK_WIDGET (cmap_lutcurve), 256, 256);

	gtk_widget_set_events (cmap_bccurve, GDK_EXPOSURE_MASK);
	gtk_widget_set_events (cmap_lutcurve, GDK_EXPOSURE_MASK);

	hsep = MakeHsep (cmap_window,vbox1,"hseparator2",-1,-1);

	/* Option menus */

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox3);
	gtk_object_set_data_full (GTK_OBJECT (cmap_window), "hbox3", hbox3, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, TRUE, 8);

	label7 = MakeLabel (cmap_window,hbox3,"  Log compression  ","cmap_label7",-1,-1);
	cmap_logviz_menu = Make_Optionmenu (cmap_window, hbox3,"logviz_cmap",-1,-1);
	cmap_logviz_menu_menu = gtk_menu_new ();
	glade_menuitem1 = Option_menuitem (cmap_logviz_menu_menu, GTK_SIGNAL_FUNC(cmap_logviz_handler), "Off", (gpointer) &menuopt0);
	glade_menuitem1 = Option_menuitem (cmap_logviz_menu_menu, GTK_SIGNAL_FUNC(cmap_logviz_handler), "Log data", (gpointer) &menuopt1);
	glade_menuitem1 = Option_menuitem (cmap_logviz_menu_menu, GTK_SIGNAL_FUNC(cmap_logviz_handler), "Log abs data", (gpointer) &menuopt2);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (cmap_logviz_menu), cmap_logviz_menu_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU(cmap_logviz_menu), cmap_use_logviz);

	vsep = MakeVsep (cmap_window,hbox3,"vseparator1",-1,-1);

	label6 = MakeLabel (cmap_window,hbox3,"    Predefined curves  ","cmap_label6",-1,-1);
	predef_cmap = Make_Optionmenu (cmap_window,hbox3,"predef_cmap",-1,-1);
	predef_cmap_menu = gtk_menu_new ();
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Default", (gpointer) &menuopt0);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Grayscale", (gpointer) &menuopt1);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Negative", (gpointer) &menuopt2);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "RGB simple", (gpointer) &menuopt3);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "RGB w/black", (gpointer) &menuopt4);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "NIH Fire", (gpointer) &menuopt5);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Rainbow", (gpointer) &menuopt6);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Rainbow w/black", (gpointer) &menuopt7);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Terra", (gpointer) &menuopt8);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Plus-Minus", (gpointer) &menuopt9);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Doppler US", (gpointer) &menuopt10);
	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "Saturation", (gpointer) &menuopt11);
//	glade_menuitem1 = Option_menuitem (predef_cmap_menu, GTK_SIGNAL_FUNC(cmap_optionmenu_handler), "From Curves", (gpointer) &menuopt11);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (predef_cmap), predef_cmap_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU(predef_cmap), active_cmap);

	vsep = MakeVsep (cmap_window,hbox3,"vseparator1",-1,-1);
	cmap_contour_button = Checkbutton_in_table (cmap_window,hbox3,"Contour Lines, density:", "histeq_loc_button", -1,-1, on_cmap_contourbutton_clicked);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cmap_contour_button), cmap_use_contours);

	cmap_ncontour_adj = GTK_ADJUSTMENT(gtk_adjustment_new (12, 2, 100, 1, 10, 0));
	cmap_ncontour = MakeSpinbutton(cmap_ncontour_adj, cmap_window, hbox3, "cmap_nontour_spin", -1,-1);
  
	/* Buttons */

	hsep = MakeHsep (cmap_window,vbox1,"hseparator1",-1,-1);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (cmap_window), "hbox2", hbox2,(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, TRUE, 8);

	cmap_create = Button_in_table (cmap_window,hbox2,"Create","cmap_cre",-1,-1,GTK_SIGNAL_FUNC (on_cmap_create_clicked));
	cmap_ok = Button_in_table (cmap_window,hbox2,"Apply","cmap_ok",-1,-1,GTK_SIGNAL_FUNC (on_cmap_ok_clicked));
	cmap_reset = Button_in_table (cmap_window,hbox2,"Reset","cmap_reset",-1,-1,GTK_SIGNAL_FUNC (on_cmap_reset_clicked));
	cmap_cancel = Button_in_table (cmap_window,hbox2,"Dismiss","cmap_cancel",-1,-1,NULL);

	gtk_signal_connect (GTK_OBJECT (cmap_window), "destroy",
                      GTK_SIGNAL_FUNC (on_cmap_cancel_clicked), cmap_window);
	gtk_signal_connect (GTK_OBJECT (cmap_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_cmap_cancel_clicked), cmap_window);
	gtk_signal_connect (GTK_OBJECT (bc_bright_adj), "value_changed",
                      GTK_SIGNAL_FUNC (bc_go), &menuopt0);
	gtk_signal_connect (GTK_OBJECT (bc_contr_adj), "value_changed",
                      GTK_SIGNAL_FUNC (bc_go), &menuopt1);
	gtk_signal_connect (GTK_OBJECT (bc_gamma_adj), "value_changed",
                      GTK_SIGNAL_FUNC (bc_go), &menuopt2);
	gtk_signal_connect (GTK_OBJECT (cmap_ncontour_adj), "value_changed",
                      GTK_SIGNAL_FUNC (cmap_ncontour_changed), &menuopt1);

	gtk_signal_connect (GTK_OBJECT (cmap_bccurve), "expose_event", GTK_SIGNAL_FUNC (on_cmap_bcexpose_event), NULL);
	gtk_signal_connect (GTK_OBJECT (cmap_bccurve), "configure_event", GTK_SIGNAL_FUNC (on_cmap_bcconfigure_event), NULL);
	gtk_signal_connect (GTK_OBJECT (cmap_lutcurve), "expose_event", GTK_SIGNAL_FUNC (on_cmap_lutexpose_event), NULL);
	gtk_signal_connect (GTK_OBJECT (cmap_lutcurve), "configure_event", GTK_SIGNAL_FUNC (on_cmap_lutconfigure_event), NULL);


	cmap_window_open = 1;
	gtk_widget_show (cmap_window);
}


/* Create the UI, start the process */

void cmap_main ()
{

	init_cmap_lut ();
	if (!cmap_window_open)
		create_cmap_window ();

}  






void bright_contrast_main ()
{


}

/***************************************************************************************************/


