


/*
 *	VIZ3d: 3D surface viewer (external cimage_sv) and fallback viewer
 */


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "application.h"
#include "backgnd.h"
#include "callbacks.h"
#include "defs.h"
#include "dlgbox.h"
#include "fileio.h"
#include "filter.h"
#include "findclust.h"
#include "functions.h"
#include "graphdsp.h"
#include "gtkhelpers.h"
#include "imgbuf.h"
#include "interface.h"
#include "morphop.h"
#include "roi.h"
#include "secondwnd.h"
#include "statop.h"
#include "special1.h"
#include "tiffstuff.h"
#include "viz.h"



/***************************************************************************************************/
/*                                                                                                 */
/*     ***    ****        ***    *   *   ****    *****      ****    *        ***    *****          */
/*    *   *   *   *      *   *   *   *   *   *   *          *   *   *       *   *     *            */
/*        *   *   *      *       *   *   *   *   *          *   *   *       *   *     *            */
/*     ***    *   *       ***    *   *   ****    ****       ****    *       *   *     *            */
/*        *   *   *          *   *   *   * *     *          *       *       *   *     *            */
/*    *   *   *   *      *   *   *   *   *  *    *          *       *       *   *     *            */
/*     ***    ****        ***     ***    *   *   *          *       *****    ***      *

	Also a secondary window function:
	Draw one slice as 3D surface plot
	Note: This function relies on the external standalone viewer "imager_sv" and tries to
	use the external viewer preferntially. If this fails (imager_sv does not exist or returns
	an error code), then we revert to the built-in fallback viewer that is implemented here.

****************************************************************************************************/


GtkWidget* surfwnd;
GtkWidget* surfdraw;
GtkAdjustment* surf_azimuth_adj;
GtkAdjustment* surf_elev_adj;
GtkAdjustment* surf_zscale_adj;
GtkAdjustment* surf_subsamp_adj;
GtkAdjustment* surf_lv_elev_adj;
GtkAdjustment* surf_lv_azim_adj;
GtkAdjustment* surf_lv_spec_adj;

image_type* surf_img;
image_type base3d_img;
image_type base3d_rgb;

struct
{
	float x,y,z;
	float elevation, azimuth;
	float len;
	float specular;
}
lightvector;

/************************************************************/


#define LVLEN	20.0

void calc_lightvector()		/* compute x,y,z,l from elev & azim */
{

	lightvector.z = LVLEN*sin(lightvector.elevation);	/* WHY sin not cos?? */
	lightvector.x = LVLEN*sin(lightvector.elevation)*sin(lightvector.azimuth);
	lightvector.y = -LVLEN*sin(lightvector.elevation)*cos(lightvector.azimuth);
	lightvector.len = sqrt(SQR(lightvector.x)+SQR(lightvector.y)+SQR(lightvector.z));
}


/* The fallback viewer uses the colomap that was last used. Default is gray */

rgbtriplet clut (float grayval)
{
rgbtriplet color;
int igrayval;	/* switch to integer arithmetic */

	igrayval = (int)(0.5+255.0*grayval);
	if (igrayval<0) igrayval=0; else if (igrayval>255) igrayval=255;

	color = cmap (igrayval, 0, 255);

	return color;
}




/* Make_base creates the elevation base map z(x,y) from the image
   by scaling it in Z direction and subsampling it as requested.
   It also provides azimuth rotation. The value range is 0...1.
   In addition a false-color RGB image is created. This saves time
   for colormap lookup during rendering. */


void make_base (image_type* img, int zslice, int intvl, float rotang)
{
long dim;
int x,y,bxm,bym;
float x1,y1,xm,ym;
float sinang, cosang, zfac, min,max, buf;
float* p;
rgbtriplet* p1;
rgbtriplet rgb;

	if (base3d_img.data) freebuf (&base3d_img);
	if (base3d_rgb.data) freebuf (&base3d_rgb);

	if (intvl<1) intvl=1;
	dim = (MAX(img->xmax,img->ymax)*141)/(100*intvl); /* times sqr(2) to make diagnonals fit */
	dp (2, "New dimensions made from %d,%d are %d sqr\n",img->xmax,img->ymax,dim);
	
	if (allocate_image (&base3d_img, B_FLOAT, dim,dim,1)==-1)
		fatal_memerr();
	if (allocate_image (&base3d_rgb, B_RGB, dim,dim,1)==-1)
		fatal_memerr();
	p = base3d_img.data; p1 = base3d_rgb.data;

	imgminmax (img,&min,&max);
	zfac = 1.0/(max-min);
	xm = 0.5*img->xmax; ym = 0.5*img->ymax;
	bxm = base3d_img.xmax/2; bym = base3d_img.ymax/2;
	sinang = sin(rotang*PI/180.0); cosang = cos(rotang*PI/180.0);

	for (y=0; y<base3d_img.ymax; y++)
		for (x=0; x<base3d_img.xmax; x++)
		{

			/* 2D Rotation matrix - rotate around center point */

			x1 = img->xmax-1.0-(xm + intvl*((x-bxm)*cosang - (y-bym)*sinang));
			y1 = img->ymax-1.0-(ym + intvl*((x-bxm)*sinang + (y-bym)*cosang));
			dp (3, "rotation from %d,%d to %f,%f\n",x,y,x1,y1);
			if ((x1<0) || (y1<0) || (x1>=img->xmax) || (y1>=img->ymax))
			{
				buf = 0;
				rgb = (rgbtriplet){255,255,255};
			}
			else
			{
				buf = (ireadbuf (img,x1,y1,zslice,intp_pref)-min)*zfac;
				rgb = clut(buf);
			}
			p [x + base3d_img.xmax*y] = buf;
			p1[x + base3d_rgb.xmax*y] = rgb;
		}
	make_minmax (&base3d_img);
	base3d_rgb.minval.imin=0; base3d_rgb.maxval.imax=255;
	/*secondary_display (&base3d_img,0,"Base Image");*/
}


/* Use the light vector here. Note that we need to define its length
   as well. The sign of X is wrong, i.e. X goes left. Y goes into the image.
   Thus, the observer's direction is something like X=0, Y=-5, Z=2.
   Light coming diagonally from the left would have a vector of (1,-1,1).
   Shade takes a rgb triplet and returns a GdkColor triplet */


GdkColor shade (rgbtriplet rgb, float x0, float y0, float z0, float x1, float y1, float z1, float x2, float y2, float z2)
{
float nx,ny,nz; 		/* normal vector */
float hx,hy,hz;			/* half vector for specular reflection */
float r,s,cy;			/* r=diffuse reflection, s=specular refl, cy=carryover */
GdkColor crgb;
float R,G,B;


	/* The above vertices are v1, v2, v3. Two base vectors are thus
	   A = v1-v0 and B = v2-v0. The normal vector is N = A x B */

	nx = (y1-y0)*(z2-z0) - (z1-z0)*(y2-y0);
	ny = (z1-z0)*(x2-x0) - (x1-x0)*(z2-z0);
	nz = (x1-x0)*(y2-y0) - (y1-y0)*(x2-x0);

	dp (3, "normal vector (%f,%f,%f)  ",nx,ny,nz);

	/* Normalize the normal vector, also consider the light vector. Avoid
	   dividing by small numbers. r is the scale factor for intensity -
	   The maximum intensity is given in rgb, and we attenuate by r.
	   r is the nomal vector dot light vector: r = N.L / (|N| |L|)
	   (Olfe pg. 428-429). Since r<=1, R,G,B cannot exceed 255
	   at this point. */

	r = lightvector.len * sqrt (SQR(nx)+SQR(ny)+SQR(nz));
	if (r < 1e-6)
		r=0.0;
	else
		r = fabs((-lightvector.x*nx+lightvector.y*ny+lightvector.z*nz)/r);

	R = r*(float)rgb.red;
	G = r*(float)rgb.green;
	B = r*(float)rgb.blue;


	/* If there is a specular component, we need to compute this, too.
	   We use the simplified model (cos a)**n where cos a = H.N / ( |H| |N| )
	   with H = halfvector = L+V. We assume the observer to be at (0,-5,2) */
	
	if (lightvector.specular>0.0)
	{
		hx = -lightvector.x + 0;
		hy = lightvector.y -5;
		hz = lightvector.z + 2;
		
		s = sqrt(SQR(hx)+SQR(hy)+SQR(hz))*sqrt (SQR(nx)+SQR(ny)+SQR(nz));
		if (s<1e-6)
			s=0.0;
		else
			s = lightvector.specular*pow(fabs((hx*nx+hy*ny+hz*nz)/s),20);
		
		/* Add specular reflection to the diffuse light computed above.
		   If the allowable maximum of 255 is exceeded, use the carryover
		   to distribute the intensity to the other channels. As a
		   result, the specular spots become more whitish, which looks
		   even more realistic. Then clamp R,G,B just to make sure. */
			
		cy=0;
		R += 127*s+0.5*s*rgb.red;	if (R>255) { cy+=R-255; R=255; }
		G += 127*s+0.5*s*rgb.green;	if (G>255) { cy+=G-255; G=255; }
		B += 127*s+0.5*s*rgb.blue;	if (B>255) { cy+=B-255; B=255; }
		R=MIN(255,R+cy);
		B=MIN(255,B+cy);
		G=MIN(255,G+cy);
	}
		

	/* Compute the return value. Note that GdkColor values are ushort, so
	   we need to multiply our original rgb values with 256 */

	crgb.red = (unsigned short)(0.49999+256*R);
	crgb.green = (unsigned short)(0.49999+256*G);
	crgb.blue = (unsigned short)(0.49999+256*B);


	return crgb;
}



void surface_plot (GtkWidget* draw, int zslice, float azimuth, float elevation, float zscale, int intvl)
{
long idx;
float* p;
rgbtriplet* p1;
int x,y, xw,yw,xw1;
float z00,z10,z01,z11,rx,ry, xnorm,ynorm, dx, dy;
float sinang, cosang, a, acosang;
GdkPoint quad[4];
GdkPoint triangle[3];
GdkGC *blkgc, *whtgc, *grygc, *usergc;
GdkColor color1, color2;
rgbtriplet rgb;

	a = 0.5;	/* depth scale factor (Olfe, pg 303) */
	/*azimuth += 180.0;*/
	xw1 = draw->allocation.width; xw=3*xw1/4;
	yw = draw->allocation.height;

	p = base3d_img.data; p1 = base3d_rgb.data;

	/* create drawing contexts for GDK: white, gray, black and variable */

	whtgc = gdk_gc_new (draw->window);	/* allocate mem & assign null values */
	gdk_gc_copy (whtgc, draw->style->white_gc); /* Copy some values */
	blkgc = gdk_gc_new (draw->window);	/* allocate mem & assign null values */
	gdk_gc_copy (blkgc, draw->style->black_gc); /* Copy some values */
	grygc = gdk_gc_new (draw->window);	/* allocate mem & assign null values */
	gdk_gc_copy (grygc, draw->style->white_gc); /* Copy some values */
	usergc = gdk_gc_new (draw->window);	/* allocate mem & assign null values */
	gdk_gc_copy (usergc, draw->style->black_gc); /* Copy some values */

	sinang = sin(elevation*PI/180.0); cosang = cos(elevation*PI/180.0);
	acosang = a*cosang;
	xnorm = ynorm = 1.0;
	dx = 1.0/base3d_img.xmax; dy = 1.0/base3d_img.ymax;

	gdk_draw_rectangle (draw->window, whtgc, 1,0,0,xw1,yw);


	for (y=base3d_img.ymax-2; y>=0; y--)	/* from back to front -- painter's */
	{
		xnorm=1.0; ynorm -= dy;
		for (x=base3d_img.xmax-2; x>=0; x--)
		{
			xnorm -= dx;
			
			/* The four elevations of the current quad */
			
			idx = x + base3d_img.xmax*y;
			z00 = zscale*p[idx];
			z10 = zscale*p[idx+1];					/* neighbor in x */
			z01 = zscale*p[idx + base3d_img.xmax];	/* neighbor in y */
			z11 = zscale*p[idx+base3d_img.xmax+1];	/* diagnoal neighbor */
			
			/* Set the colors - the quad gets only ONE color*/
			
			rgb = p1[idx];
		
			/* Create the oblique projection of the quad. The plot
  			   x and y coords are normalized to 0...1, and we swap
			   x and y axes over the standard (Olfe 302-303).
			   Please don't ask me why I had to swap z01 and z10. 

				Note that at this place, we can easily implement other projections,
				such as the isometric view and the axonometric projection.
				This merely involves modifying the projection equations for rx and ry. */

			rx = xnorm - ynorm*acosang;
			ry = -z00 - ynorm*acosang;
			quad[0].x = (int)(0.5+xw-rx*xw);
			quad[0].y = (int)(yw + 0.5 + ry*yw);

			rx = xnorm - (ynorm+dy)*acosang;
			ry = -z01 - (ynorm+dy)*acosang;
			quad[1].x = (int)(0.5+xw-rx*xw);
			quad[1].y = (int)(yw + 0.5 + ry*yw);
			
			rx = (xnorm+dx) - ynorm*acosang;
			ry = -z10 - ynorm*acosang;
			quad[2].x = (int)(0.5+xw-rx*xw);
			quad[2].y = (int)(yw + 0.5 + ry*yw);

			rx = (xnorm+dx) - (ynorm+dy)*acosang;
			ry = -z11 - (ynorm+dy)*acosang;
			quad[3].x = (int)(0.5+xw-rx*xw);
			quad[3].y = (int)(yw + 0.5 + ry*yw);

			
			/* First triangle. 
			   Please note that the order of the vertices
			   passed to "shade" matters. They have to create a left-hand
			   system (due to some unexplored sign flaw).
			   Also note that the gdkcolor elements are 16-bit. By
			   using the shade (scaled 0..255) we achieve the full
			   16-bit width */
			
			color1 = shade(rgb,xnorm,ynorm,z00,xnorm,ynorm+dy,z01,xnorm+dx,ynorm,z10);
			gdk_color_alloc (gdk_colormap_get_system(), &color1);
			gdk_gc_set_foreground (usergc, &color1);
			
			triangle[0]=quad[0]; triangle[1]=quad[1]; triangle[2]=quad[3];
			gdk_draw_polygon (draw->window, usergc, 1, triangle, 3);
			
			/* second triangle */
			
			color2 = shade(rgb,xnorm,ynorm,z00,xnorm,ynorm+dy,z01,xnorm+dx,ynorm,z10);
			gdk_color_alloc (gdk_colormap_get_system(), &color2);
			gdk_gc_set_foreground (usergc, &color2);
			
			triangle[0]=quad[0]; triangle[1]=quad[2]; triangle[2]=quad[3];
			gdk_draw_polygon (draw->window, usergc, 1, triangle, 3);

		}
	}

	gdk_gc_destroy (whtgc);
	gdk_gc_destroy (blkgc);
	gdk_gc_destroy (grygc);
	gdk_gc_destroy (usergc);
}


/****************************************************+

	Surface plot GUI
	
*****************************************************/



void go_plot_3d ()
{
float zscale, azimuth, elevation;
int subsamp;
int intvl;

	///predef_lut_rgb_simple(0);				This had to go with the LUT module. Actually needed?
	zscale = surf_zscale_adj->value / 100.0;
	azimuth = surf_azimuth_adj->value;
	elevation = surf_elev_adj->value;
	subsamp = surf_subsamp_adj->value;
	intvl = 1<<(subsamp-1);
	lightvector.elevation = surf_lv_elev_adj->value*PI/180.0;
	lightvector.azimuth = surf_lv_azim_adj->value*PI/180.0;
	lightvector.specular= surf_lv_spec_adj->value/100.0;
	calc_lightvector();

	if ((base3d_img.data==NULL) || (base3d_rgb.data==NULL)) 
		make_base (surf_img, cur_slice, intvl, azimuth);
	surface_plot (surfdraw, cur_slice, azimuth, elevation, zscale, intvl);

}



gboolean on_surfdraw_expose_event (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{

	go_plot_3d ();

	return FALSE;
}


gboolean on_surfdraw_button_press_event (GtkWidget *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
int x,y;
GdkModifierType state;

	gdk_window_get_pointer (widget->window, &x, &y, &state);
	dp (1, "*click* at (%d,%d), state is %04x\n",x,y,state);

  	return FALSE;
}


gboolean on_surfdraw_button_release_event (GtkWidget *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

	return FALSE;
}


gboolean on_surfdraw_configure_event (GtkWidget *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{

	go_plot_3d ();

	return FALSE;
}


void destroy_surfwnd (GtkWidget *widget, gpointer user_data)
{
	freebuf (&base3d_img); freebuf (&base3d_rgb);
	gtk_widget_destroy ( GTK_WIDGET ( surfwnd ) );
}


void on_surf_slider_changed (GtkWidget* widget, gpointer user_data)
{
int slidernum;

	slidernum= *(int*)user_data;
	if ((slidernum==menuopt1) || (slidernum==menuopt4))
	{
		freebuf (&base3d_img); freebuf (&base3d_rgb);
	}
	go_plot_3d ();

}



void create_surfplot_wnd (image_type* img, char* title)
{

GtkWidget *vbox1;
GtkWidget* hbox1;
GtkWidget* table1;
GtkWidget *s3dlabel;
GtkWidget* swbutton1;
GtkWidget* swbutton2;
GtkWidget *hseparator1, *hseparator2;
GtkWidget *surf_azimuth, *surf_elev, *surf_zscale, *surf_subsamp;
GtkWidget *surf_lv_elev, *surf_lv_azim, *surf_lv_spec;
GtkWidget *label1, *label2, *label3, *label4, *label5, *label6, *label7;
int submax,subset, subpwr2;

	surf_img = img;
	subset=1;
	subpwr2 = 1; submax=1;
	while (submax < MIN(img->xmax,img->ymax)) { submax = submax <<1; subpwr2++; }
	if (submax>8) submax=submax>>3;
	if (MAX(img->xmax,img->ymax) >=1024)
		subset=4;
	else if (MAX(img->xmax,img->ymax) >=512)
		subset=3;
	else if (MAX(img->xmax,img->ymax) >=256)
		subset=2;
		
	base3d_img.data=NULL; base3d_rgb.data=NULL;
	

	surfwnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (surfwnd), "surfwnd", surfwnd);
	gtk_window_set_title (GTK_WINDOW (surfwnd), "Surface plot");

	/* Items will be placed top to bottom into this vbox as they come */

	vbox1 = vbox_in_window (surfwnd, "s3dvbox1");

	/* A label field with a title or caption */

	s3dlabel = MakeLabel (surfwnd,vbox1,title, "s3d_label",-1,-1);

	/* the actual drawing area */

	surfdraw = gtk_drawing_area_new ();
	gtk_widget_ref (surfdraw);
	gtk_object_set_data_full (GTK_OBJECT (surfwnd), "secondaryimg", surfdraw,
                              (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (surfdraw);
	gtk_box_pack_start (GTK_BOX (vbox1), surfdraw, TRUE, TRUE, 0);
	gtk_widget_set_usize (surfdraw, 320, 200);

	/* a nice-looking hseparator and a table for the adjustments */

	hseparator1 = MakeHsep (surfwnd,vbox1, "shsep1",-1,-1);

	table1 = MakeTable (surfwnd,vbox1,"stab1",2,7);
	
	/* The adjustments with lables */
	
	label1 = MakeLabel (surfwnd,table1,"Azimuth","s3d_az",0,0);
	label2 = MakeLabel (surfwnd,table1,"Elevation","s3d_el",0,1);
	label3 = MakeLabel (surfwnd,table1,"Z scale","s3d_zs",0,2);
	label4 = MakeLabel (surfwnd,table1,"Subsampling","s3d_ssa",0,3);
	label5 = MakeLabel (surfwnd,table1,"Light vector elevation","s3d_lve",0,4);
	label6 = MakeLabel (surfwnd,table1,"Light vector azimuth","s3d_lva",0,5);
	label7 = MakeLabel (surfwnd,table1,"Specular intensity","s3d_lvs",0,6);

	surf_azimuth_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, -180, 180, 1, 10, 1));
	surf_azimuth = MakeHScale (surf_azimuth_adj, surfwnd, table1, "s3d_hscale1", 1,0);
	surf_elev_adj = GTK_ADJUSTMENT(gtk_adjustment_new (35, 10, 60, 1, 10, 1));
	surf_elev = MakeHScale (surf_elev_adj, surfwnd, table1, "s3d_hscale2", 1,1);
	surf_zscale_adj = GTK_ADJUSTMENT(gtk_adjustment_new (20, 1, 100, 1, 10, 1));
	surf_zscale = MakeHScale (surf_zscale_adj, surfwnd, table1, "s3d_hscale3", 1,2);
	surf_subsamp_adj = GTK_ADJUSTMENT(gtk_adjustment_new (subset, 1, subpwr2, 1, 1, 1));
	surf_subsamp = MakeHScale (surf_subsamp_adj, surfwnd, table1, "s3d_hscale4", 1,3);
	gtk_scale_set_digits (GTK_SCALE (surf_subsamp), 0);
	surf_lv_elev_adj = GTK_ADJUSTMENT(gtk_adjustment_new (45, 0, 90, 1, 10, 1));
	surf_lv_elev = MakeHScale (surf_lv_elev_adj, surfwnd, table1, "s3d_hscale5", 1,4);
	surf_lv_azim_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, -90, 90, 1, 10, 1));
	surf_lv_azim = MakeHScale (surf_lv_azim_adj, surfwnd, table1, "s3d_hscale6", 1,5);
	surf_lv_spec_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, 0, 100, 1, 10, 1));
	surf_lv_spec = MakeHScale (surf_lv_spec_adj, surfwnd, table1, "s3d_hscale7", 1,6);
	
	/* a hbox to contain the buttons */

	hseparator2 = MakeHsep (surfwnd,vbox1, "shsep2",-1,-1);
	
	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (surfwnd), "swhbox1", hbox1,
                              (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	/* gtk_container_add (GTK_CONTAINER (vbox1), hbox1); */
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 2);
 
 	swbutton1 = Button_in_table (surfwnd,hbox1,"[button]","swbutton1",-1,-1,NULL);
 	swbutton2 = Button_in_table (surfwnd,hbox1,"Dismiss","swbutton2",-1,-1,destroy_surfwnd);
 
	gtk_widget_set_events (surfdraw, GDK_EXPOSURE_MASK
										  | GDK_LEAVE_NOTIFY_MASK
										  | GDK_BUTTON_PRESS_MASK
										  | GDK_BUTTON_RELEASE_MASK
										  | GDK_POINTER_MOTION_MASK
										  | GDK_POINTER_MOTION_HINT_MASK
										  | GDK_KEY_PRESS_MASK);


	gtk_signal_connect (GTK_OBJECT (surf_azimuth_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt1);
	gtk_signal_connect (GTK_OBJECT (surf_elev_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt2);
	gtk_signal_connect (GTK_OBJECT (surf_zscale_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt3);
	gtk_signal_connect (GTK_OBJECT (surf_subsamp_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt4);
	gtk_signal_connect (GTK_OBJECT (surf_lv_elev_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt5);
	gtk_signal_connect (GTK_OBJECT (surf_lv_azim_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt6);
 	gtk_signal_connect (GTK_OBJECT (surf_lv_spec_adj), "value_changed",
                      GTK_SIGNAL_FUNC (on_surf_slider_changed), &menuopt7);
 
	gtk_signal_connect (GTK_OBJECT (surfdraw), "expose_event",
                    	GTK_SIGNAL_FUNC (on_surfdraw_expose_event),NULL);
	gtk_signal_connect (GTK_OBJECT (surfdraw), "button_press_event",
                    	GTK_SIGNAL_FUNC (on_surfdraw_button_press_event),NULL);
	gtk_signal_connect (GTK_OBJECT (surfdraw), "button_release_event",
                    	GTK_SIGNAL_FUNC (on_surfdraw_button_release_event),NULL);
	gtk_signal_connect (GTK_OBJECT (surfdraw), "configure_event",
                    	GTK_SIGNAL_FUNC (on_surfdraw_configure_event),NULL);
	gtk_signal_connect (GTK_OBJECT (surfwnd), "destroy",
                    	GTK_SIGNAL_FUNC (destroy_surfwnd),secondwnd);


	sec_statusbar_was_used=0;
	gtk_widget_show (surfwnd);

	gdk_window_set_cursor (surfdraw->window, crosshair_cursor);

}




int start_imager_sv (const char* filename)
{
int pid, rtncode;
char* argv[6];
char loadparm[10];
char dbgparm[10];
char dbglvl[10];

	rtncode=0;
	strcpy (loadparm, "-l");
	strcpy (dbgparm, "-d");
	strcpy (dbglvl, "1");		/* Rather than inheriting the debug level, create less clutter */

	argv[0] = imager_sv_path;
	argv[1]= loadparm;
	argv[2] =filename;
	argv[3]= dbgparm;
	argv[4] =dbglvl;
	argv[5] = NULL;		/* Arg list must end with a NULL pointer */

	dp (2, "cimage_sv: Forking...");

	pid = fork();

	/* This is the first statement executed by the original and the forked process */
	
	dp (1, "   fork PID %d\n", pid);
	if (pid==-1)
	{
		statusbar_display ("Error: Could not fork ()");
		dp (0, "Error: could not fork\n");
		return -1;
	}
	
	/* Note that at this point the forked process and the parent
	   process have different values for pid. The parent has the
	   regular pid of the child, thus can perform "waitpid" further down.
	   The child thinks it is pid==0 and thus enters the if-statement
	   that follows, and terminates on the execl statement */
	
	if (pid==0)
	{

		dp (1, "cimage (forked): starting backend, pid=%d\n", pid);
		rtncode = execv (imager_sv_path, argv);
		
		/* The following statements will not be executed 
		   if execl was successful */
		
		dp (0, "cimage (forked): This message should not appear. Failure %d\n", rtncode);
		exit (-1);
	}

	return pid;		/* pid of the forked process == pid of slave process */

}




/* In the main driver, we select which viewer to use, the external cimage_sv
   or the built-in viewer. */

void surface_draw_main ()
{
int fallback, err;
gchar *fn;
struct stat statresult;		/* to check if a file/program exists */


	fn = create_tempfn ();
	fallback = 0;
	if ((strlen(undo_path)==0) || strlen (imager_sv_path)==0)
	{
		statusbar_display ("Path(s) not specified. Using fallback viewer.");
		fallback = 1;
	}

	/* Copy the present slice to the temp directory (save as TIFF)
	   Try packbits compression */
	
	if (!fallback)
	{
		err=tiffwrite (fn, &mainimg, 2, "cimage TEMP file", cur_slice, cur_slice);
		if (err < 0) 
			fallback = 1;
		else
			dp (1, "3D viewer: Successfully generated temporary file %s\n", fn);
	}
	
	/* Temp file has been written, now fork the cimage_sv process */
	
	if (!fallback)
	{
		err=stat (imager_sv_path, &statresult);
		if (!err)
		{
			err = start_imager_sv (fn);
			if (err<0) 
				fallback = 1;
			else
				statusbar_display ("External 3D viewer started.");
		}
		else
			fallback=1;
	}


	free (fn);
	if (fallback) create_surfplot_wnd (&mainimg, "3D Fallback Viewer");
}	
	

